//Sample function to create a php based nagzilla client
//(c)2009 Hurricane Labs
//by: Billford

function shout_it_nagzilla ($room,$message,$poster)
 {

$host = "nagzilla_server";
$port = 49776;
$timeout = 30;

$mess_too = $poster . " shouted ".$message;
$send = "room^".$room."^".$mess_too;

$nag=fsockopen($host,$port,$errnum,$errstr,$timeout);

if (!is_resource($nag)) {
        exit("Connection to Nagzilla Server failed ".$errnum . " ".$errstr);
}else {
        fputs($nag,$send);
}

fclose($nag);

}
